/***************************************************************************
 *   Copyright (C) 2008  Antony Martin <martin(dot)antony(at)yahoo(dot)fr> *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 11 mars 2007
   derniere modification le 23/11/2008 */

#ifndef SCETEXTURE_H
#define SCETEXTURE_H

#include <stdarg.h>
#include <SCE/utils/SCEList.h>
#include <SCE/core/SCECTexture.h>
#include <SCE/core/SCECFramebuffer.h>

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * \ingroup texture
 * @{
 */

/* filtres de redimensionnement */
/* NOTE: noms constantes a revoir (conflit avec SCE_TEX_NEAREST & cie) */
/* TODO: qu'est-ce que ces constantes foutent ici ?? */
#define SCE_FILTER_NEAREST      ILU_NEAREST
#define SCE_FILTER_LINEAR       ILU_LINEAR
#define SCE_FILTER_BILINEAR     ILU_BILINEAR
#define SCE_FILTER_BOX          ILU_SCALE_BOX
#define SCE_FILTER_TRIANGLE     ILU_SCALE_TRIANGLE
#define SCE_FILTER_BELL         ILU_SCALE_BELL
#define SCE_FILTER_BSPLINE      ILU_SCALE_BSPLINE
#define SCE_FILTER_LANCZOS3     ILU_SCALE_LANCZOS3
#define SCE_FILTER_MITCHELL     ILU_SCALE_MITCHELL

/* types supplementaires acceptables pour la creation d'une texture */
/* NOTE: ces constantes doivent toutes etre differentes
         de celles de CFramebuffer... */
#define SCE_RENDER_COLOR 1
#define SCE_RENDER_COLOR_CUBE 2
#define SCE_RENDER_DEPTH 3
#define SCE_RENDER_DEPTH_CUBE 4

/* defini la face de la cubemap sur laquelle rendre */
#define SCE_RENDER_POSX 0
#define SCE_RENDER_NEGX 1
#define SCE_RENDER_POSY 2
#define SCE_RENDER_NEGY 3
#define SCE_RENDER_POSZ 4
#define SCE_RENDER_NEGZ 5

#define SCE_MAX_TEXTURE_UNITS 32


/** \copydoc sce_stexture */
typedef struct sce_stexture SCE_STexture;
/**
 * \brief A SCE texture that stores many informations about one or more textures
 */
struct sce_stexture
{
    SCE_CFramebuffer *fb[6]; /**< Integrated frame buffers (6 for cubemaps) */
    SCE_CTexture *tex;       /**< Core texture */
    unsigned int unit;       /**< Texture unit */
    SCE_TMatrix4 matrix;     /**< Texture matrix */
    int type;                /**< Type of the texture */
    int used;                /**< Is texture used for rendering? */
    int toremove;            /**< Internal use (like everything lol) */
#if SCE_LIST_ITERATOR_NO_MALLOC
    SCE_SListIterator iterator;
#endif
    SCE_SListIterator *it;   /**< Own iterator */
};

/** @} */

/* initialise le gestionnaire de textures */
int SCE_Init_Texture (void);
/* quitte le gestionnaire de textures */
void SCE_Quit_Texture (void);

/* defini la texture active */
void SCE_Texture_Bind (SCE_STexture*);

/* cree une texture en prenant son type */
SCE_STexture* SCE_Texture_Create (SCEenum, int, int);
/* supprime une texture */
void SCE_Texture_Delete (SCE_STexture*);
void SCE_Texture_Delete_ (void);

/* defini les filtrages */
void SCE_Texture_SetFilter (SCE_STexture*, SCEint);
void SCE_Texture_Pixelize (SCE_STexture*, int);

/* permet de definir un parametre a une texture */
void SCE_Texture_SetParam (SCE_STexture*, SCEenum, SCEint);
void SCE_Texture_SetParamf (SCE_STexture*, SCEenum, SCEfloat);

void SCE_Texture_SetUnit (SCE_STexture*, unsigned int);
unsigned int SCE_Texture_GetUnit (SCE_STexture*);

float* SCE_Texture_GetMatrix (SCE_STexture*);

/* force certaines valeurs (voir SCECTexture.c) */
void SCE_Texture_ForcePixelFormat (int, int);
void SCE_Texture_ForceType (int, int);
void SCE_Texture_ForceFormat (int, int);

/* renvoie le type de la texture (peut etre de type RENDER_<...>) */
int SCE_Texture_GetType (SCE_STexture*);
/* renvoie le type de la texture coeur (target opengl) */
int SCE_Texture_GetCType (SCE_STexture*);

/* renvoie la texture de.. la texture :D */
SCE_CTexture* SCE_Texture_GetCTexture (SCE_STexture*);
SCE_CTexture* SCE_Texture_GetCTexture_ (void);

/* retourne les dimensions d'une texture */
int SCE_Texture_GetWidth (SCE_STexture*, int, int);
int SCE_Texture_GetHeight (SCE_STexture*, int, int);

/* construit la texture */
int SCE_Texture_Build (SCE_STexture*, int);
int SCE_Texture_Build_ (int);

/* met a jour les donnees de la texture */
int SCE_Texture_Update (SCE_STexture*);
int SCE_Texture_Update_ (void);

/* cree une texture a partir d'un ou plusieurs fichiers */
SCE_STexture* SCE_Texture_LoadArg (int, int, int, int, va_list);
SCE_STexture* SCE_Texture_Load (int, int, int, int, ...);

/* ajoute une texture de rendu */
int SCE_Texture_AddRenderCTexture (SCE_STexture*, int, SCE_CTexture*);
int SCE_Texture_AddRenderTexture (SCE_STexture*, int, SCE_STexture*);

/* blit une texture sur une autre */
void SCE_Texture_Blit (SCE_SIntRect*, SCE_STexture*,
                       SCE_SIntRect*, SCE_STexture*);
void SCE_Texture_Blitf (SCE_SFloatRect*, SCE_STexture*,
                        SCE_SFloatRect*, SCE_STexture*);

/* ajoute/enleve une texture a la pile des textures utilisees */
void SCE_Texture_Use (SCE_STexture*);

void SCE_Texture_Flush (void);

void SCE_Texture_BeginLot (void);
void SCE_Texture_EndLot (void);

#if 0
void SCE_Texture_Mark (void);
void SCE_Texture_Restore (void);
#endif

/* defini la texture dans laquelle effectuer le rendu
   (sous condition que la texture soit de type "render texture") */
void SCE_Texture_RenderTo (SCE_STexture*, SCEuint);
void SCE_Texture_RenderTo_ (SCEuint);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* guard */
